#ifndef BLENDSTATE_H
#define BLENDSTATE_H

#include "D3D11_Core.h"

/// <summary>
/// Takes care of managing a blend state
/// </summary>
class BlendState
{
  public:
    /// <summary>
    /// Creates a blend state
    /// </summary>
    /// <param name="desc">
    /// configuration values that describe the state to create
    /// </param>
    /// <param name="graphics">
    /// D3D11 instance to create the state for
    /// </param>
    /// <returns>
    /// pointer to the blend state
    /// NULL if an error occurred
    /// </returns>
    static BlendState* Create(D3D11_BLEND_DESC desc,D3D11_Core& graphics);
    
    /// <summary>
    /// Retrieves the current blend state
    /// <summary>
    /// <param name="context">
    /// D3D11 context to query the blend state of
    /// </param>
    static BlendState* GetCurrent(ID3D11DeviceContext* context);
    
    ~BlendState();
    
    /// <summary>
    /// Setups the blend state to be used for rendering
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the state to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
  private:
    BlendState(ID3D11BlendState* state);
    BlendState(ID3D11BlendState* state,float factors[4],UINT mask);
    
    // disabled
    BlendState();
    BlendState(const BlendState& cpy);
    BlendState& operator=(const BlendState& cpy);
    
    /// <summary>
    /// the blend state
    /// </summary>
    ID3D11BlendState* m_state;
    
    /// <summary>
    /// blend factors for rgba
    /// </summary>
    float m_factors[4];
    
    /// <summary>
    /// sample mask
    /// </summary>
    UINT m_mask;
};

#endif /* BLENDSTATE_H */
