#ifndef INDEXBUFFER_H
#define INDEXBUFFER_H

#include "Graphics/D3D11_Core.h"

/// <summary>
/// Base class for index buffers
/// </summary>
class IndexBuffer
{
  public:
    virtual ~IndexBuffer();
    
    /// <summary>
    /// Setup the index buffer for usage with the specified context
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the buffer to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
  protected:
    IndexBuffer(ID3D11Buffer* buffer);
    
    /// <summary>
    /// Creates an index buffer
    /// </summary>
    /// <param name="graphics">
    /// D3D11 instance to create the index buffer for
    /// </param>
    /// <param name="num">
    /// number of entries in the buffer
    /// </param>
    /// <param name="size">
    /// number of bytes in 1 entry
    /// </param>
    /// <param name="data">
    /// initial data for the index buffer
    /// </param>
    static ID3D11Buffer* CreateBuffer(D3D11_Core& graphics,UINT num,UINT size,
      const void* data);
    
    /// <summary>
    /// Retrieves the format of the index buffer
    /// </summary>
    /// <returns>
    /// format of the index buffer
    /// </returns>
    virtual DXGI_FORMAT GetFormat() = 0;
    
  private:
    // disabled
    IndexBuffer();
    IndexBuffer(const IndexBuffer& cpy);
    IndexBuffer& operator=(const IndexBuffer& cpy);
    
    /// <summary>
    /// D3D11 index buffer
    /// </summary>
    /// <remarks>
    /// base class releases this in its destructor if not NULL
    /// </remarks>
    ID3D11Buffer* m_buffer;
};

#endif /* INDEXBUFFER_H */
