#ifndef INDEXBUFFER16_H
#define INDEXBUFFER16_H

#include "IndexBuffer.h"

/// <summary>
/// Index buffer for 16bit long entries
/// </summary>
class IndexBuffer16 : public IndexBuffer
{
  public:
    /// <summary>
    /// Creates an index buffer
    /// </summary>
    /// <param name="graphics">
    /// D3D11 instance to create the index buffer for
    /// </param>
    /// <param name="num">
    /// number of entries in the buffer
    /// </param>
    /// <param name="data">
    /// initial data for the index buffer
    /// </param>
    static IndexBuffer16* Create(D3D11_Core& graphics,UINT num,
      const WORD* data);
    
    /// <summary>
    /// Cleans up the index buffer
    /// </summary>
    ~IndexBuffer16();
    
  protected:
    /// <summary>
    /// Retrieves the format of the index buffer
    /// </summary>
    /// <returns>
    /// format of the index buffer
    /// </returns>
    DXGI_FORMAT GetFormat();
    
  private:
    // disabled
    IndexBuffer16();
    IndexBuffer16(const IndexBuffer16& cpy);
    IndexBuffer16& operator=(const IndexBuffer16& cpy);
    
    /// <summary>
    /// Creates an index buffer
    /// </summary>
    /// <param name="buffer">
    /// d3d11 index buffer
    /// </param>
    IndexBuffer16(ID3D11Buffer* buffer);
};

#endif /* INDEXBUFFER16_H */
