#ifndef INSTANCEBUFFER_H
#define INSTANCEBUFFER_H

#include "Graphics/Buffers/VertexBuffer.h"

class InstanceBuffer : public VertexBuffer
{
  public:
    /// <summary>
    /// Creates an instance buffer
    /// </summary>
    /// <param name="graphics">
    /// D3D11 instance to create the buffer for
    /// </param>
    /// <param name="num">
    /// number of entries in the buffer
    /// </param>
    /// <param name="size">
    /// number of bytes in 1 entry
    /// </param>
    /// <param name="data">
    /// initial data for the instance buffer
    /// </param>
    /// <returns>
    /// pointer to the buffer on success
    /// NULL if an error occurred
    /// </returns>
    static InstanceBuffer* Create(D3D11_Core& graphics,UINT num,UINT size,
      const void* data);
    
    ~InstanceBuffer();
    
    /// <summary>
    /// Replaces the data in the instance buffer
    /// <summary>
    /// <param name="context">
    /// D3D11 context to update the buffer in
    /// </param>
    /// <param name="data">
    /// data to store in the instance buffer
    /// <remarks>
    /// this is a complete replacement, so this must be the same size as
    /// specified to the Create call
    /// </remarks>
    /// </param>
    /// <returns>
    /// true if the update occurred successfully
    /// false otherwise
    /// </returns>
    bool Update(ID3D11DeviceContext* context,const void* data);
    
  protected:
    InstanceBuffer(ID3D11Buffer* buffer,UINT size,UINT num);
    
    /// <summary>
    /// Retrieves the stride between elements
    /// </summary>
    /// <returns>
    /// number of bytes between each element in the vertex buffer
    /// </returns>
    UINT GetStride() const;
    
  private:
    // disabled
    InstanceBuffer(const InstanceBuffer& cpy);
    InstanceBuffer& operator=(const InstanceBuffer& cpy);
    
    /// <summary>
    /// number of bytes per element
    /// </summary>
    UINT m_stride;
    
    /// <summary>
    /// total number of bytes in the buffer (needed for Update)
    /// </summary>
    UINT m_num_bytes;
};

#endif /* INSTANCEBUFFER_H */
