#ifndef VERTEXBUFFER_POSITION_H
#define VERTEXBUFFER_POSITION_H

#include "Graphics/Buffers/VertexBuffer.h"
#include "Graphics/VertexDeclarations/Vertex_Position.h"

/// <summary>
/// Vertex buffer whose elements are position values
/// </summary>
class VertexBuffer_Position : public VertexBuffer
{
  public:
    /// <summary>
    /// Creates a vertex buffer
    /// </summary>
    /// <param name="graphics">
    /// D3D11 instance to create the vertex buffer for
    /// </param>
    /// <param name="num">
    /// number of entries in the buffer
    /// </param>
    /// <param name="data">
    /// initial data for the vertex buffer
    /// </param>
    /// <returns>
    /// pointer to the vertex buffer on success
    /// NULL if an error occurred
    /// </returns>
    static VertexBuffer_Position* Create(D3D11_Core& graphics,UINT num,
      const Vertex_Position* data);
    
    /// <summary>
    /// Cleans up the vertex buffer
    /// </summary>
    ~VertexBuffer_Position();
    
  protected:
    /// <summary>
    /// Retrieves the stride between elements
    /// </summary>
    /// <returns>
    /// number of bytes between each element in the vertex buffer
    /// </returns>
    UINT GetStride() const;
    
  private:
    // disabled
    VertexBuffer_Position();
    VertexBuffer_Position(const VertexBuffer_Position& cpy);
    VertexBuffer_Position& operator=(
      const VertexBuffer_Position& cpy);
    
    /// <summary>
    /// Creates a vertex buffer
    /// </summary>
    /// <param name="buffer">
    /// d3d11 vertex buffer
    /// </param>
    /// <param name="num">
    /// number of entries in the buffer
    /// </param>
    VertexBuffer_Position(ID3D11Buffer* buffer,UINT num);
};

#endif /* VERTEXBUFFER_POSITION_H */
