#ifndef VERTEXBUFFER_POSITIONTEXTURE_H
#define VERTEXBUFFER_POSITIONTEXTURE_H

#include "Graphics/Buffers/VertexBuffer.h"
#include "Graphics/VertexDeclarations/Vertex_PositionTexture.h"

/// <summary>
/// Vertex buffer whose elements are position and texture values
/// </summary>
class VertexBuffer_PositionTexture : public VertexBuffer
{
  public:
    /// <summary>
    /// Creates a vertex buffer
    /// </summary>
    /// <param name="graphics">
    /// D3D11 instance to create the vertex buffer for
    /// </param>
    /// <param name="num">
    /// number of entries in the buffer
    /// </param>
    /// <param name="data">
    /// initial data for the vertex buffer
    /// </param>
    static VertexBuffer_PositionTexture* Create(D3D11_Core& graphics,UINT num,
      const Vertex_PositionTexture* data);
    
    /// <summary>
    /// Cleans up the vertex buffer
    /// </summary>
    ~VertexBuffer_PositionTexture();
    
  protected:
    /// <summary>
    /// Retrieves the stride between elements
    /// </summary>
    /// <returns>
    /// number of bytes between each element in the vertex buffer
    /// </returns>
    UINT GetStride() const;
    
  private:
    // disabled
    VertexBuffer_PositionTexture();
    VertexBuffer_PositionTexture(const VertexBuffer_PositionTexture& cpy);
    VertexBuffer_PositionTexture& operator=(
      const VertexBuffer_PositionTexture& cpy);
    
    /// <summary>
    /// Creates a vertex buffer
    /// </summary>
    /// <param name="buffer">
    /// d3d11 vertex buffer
    /// </param>
    /// <param name="num">
    /// number of entries in the buffer
    /// </param>
    VertexBuffer_PositionTexture(ID3D11Buffer* buffer,UINT num);
};

#endif /* VERTEXBUFFER_POSITIONTEXTURE_H */
