#ifndef DEPTH_STENCIL_STATE_H
#define DEPTH_STENCIL_STATE_H

#include "D3D11_Core.h"

/// <summary>
/// Takes care of managing a depth stencil state
/// </summary>
class DepthStencilState
{
  public:
    /// <summary>
    /// Creates a depth stencil state
    /// </summary>
    /// <param name="desc">
    /// configuration values that describe the state to create
    /// </param>
    /// <param name="graphics">
    /// D3D11 instance to create the state for
    /// </param>
    /// <returns>
    /// pointer to the depth stencil state
    /// NULL if an error occurred
    /// </returns>
    static DepthStencilState* Create(D3D11_DEPTH_STENCIL_DESC desc,
      D3D11_Core& graphics);
    
    /// <summary>
    /// Retrieves the current depth stencil state
    /// <summary>
    /// <param name="context">
    /// D3D11 context to query the depth stencil state of
    /// </param>
    static DepthStencilState* GetCurrent(ID3D11DeviceContext* context);
    
    ~DepthStencilState();
    
    /// <summary>
    /// Setups the depth stencil state to be used for rendering
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the state to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
  private:
    DepthStencilState(ID3D11DepthStencilState* state);
    DepthStencilState(ID3D11DepthStencilState* state,UINT m_ref);
    
    // disabled
    DepthStencilState();
    DepthStencilState(const DepthStencilState& cpy);
    DepthStencilState& operator=(const DepthStencilState& cpy);
    
    /// <summary>
    /// depth stencil state
    /// </summary>
    ID3D11DepthStencilState* m_state;
    
    /// <summary>
    /// reference value
    /// </summary>
    UINT m_ref;
};

#endif /* DEPTH_STENCIL_STATE_H */
