#ifndef TEXTURE_H
#define TEXTURE_H

#include <d3d11.h>

class ContentManager;

/// <summary>
/// Takes care of managing a 2D texture
/// </summary>
class Texture
{
  public:
#ifndef PUBLIC_HEADER
    Texture(ID3D11ShaderResourceView* texture);
    
    /// <summary>
    /// Retrieves the texture
    /// </summary>
    /// <returns>
    /// const pointer to the texture resource view
    /// </returns>
    ID3D11ShaderResourceView* GetTexture() const;
#endif /* PUBLIC_HEADER */
    
  private:
    // disabled
    Texture();
    Texture(const Texture& cpy);
    Texture& operator=(const Texture& cpy);
    
    /// <remarks>
    /// private so that ContentManager has to be used to clean it up
    /// </remarks>
    ~Texture();
    
    // so a ContentManager instance can deallocate at the appropriate time
    friend ContentManager;
    
    /// <summary>
    /// D3D11 version of the texture
    /// </summary>
    ID3D11ShaderResourceView* m_texture;
};

#endif /* TEXTURE_H */
