#ifndef TEXTURESAMPLER_H
#define TEXTURESAMPLER_H

#include "D3D11_Core.h"

/// <summary>
/// Takes care of managing a texture sampler
/// </summary>
class TextureSampler
{
  public:
    /// <summary>
    /// Creates a texture sampler
    /// </summary>
    /// <param name="desc">
    /// configuration values that describe the sampler to create
    /// </param>
    /// <param name="graphics">
    /// D3D11 instance to create the sampler for
    /// </param>
    /// <returns>
    /// pointer to the sampler
    /// NULL if an error occurred
    /// </returns>
    static TextureSampler* Create(D3D11_SAMPLER_DESC desc,
      D3D11_Core& graphics);
    
    ~TextureSampler();
    
#ifndef PUBLIC_HEADER
    /// <summary>
    /// Retrieves the texture sampler
    /// </summary>
    /// <returns>
    /// const pointer to the texture sampler
    /// </returns>
    ID3D11SamplerState* GetTextureSampler() const;
#endif /* PUBLIC_HEADER */
    
  private:
    TextureSampler(ID3D11SamplerState* sampler);
    
    // disabled
    TextureSampler();
    TextureSampler(const TextureSampler& cpy);
    TextureSampler& operator=(const TextureSampler& cpy);
    
    /// <summary>
    /// the sampler
    /// </summary>
    ID3D11SamplerState* m_sampler;
};

#endif /* TEXTURESAMPLER_H */
