#include "Graphics/BlendState.h"
#include "log.h"

BlendState* BlendState::Create(D3D11_BLEND_DESC desc,D3D11_Core& graphics)
{
  ID3D11BlendState* state = NULL;
  if (FAILED(graphics.GetDevice()->CreateBlendState(&desc,&state)))
  {
    log_print("Failed to create blend state\n");
    return NULL;
  }
  
  return new BlendState(state);
}

BlendState* BlendState::GetCurrent(ID3D11DeviceContext* context)
{
  ID3D11BlendState* state      = NULL;
  float             factors[4] = { 0,0,0,0 };
  UINT              mask       = 0;
  
  context->OMGetBlendState(&state,factors,&mask);
  
  return new BlendState(state,factors,mask);
}

BlendState::BlendState(ID3D11BlendState* state)
:m_state(state),
 m_mask(0xffffffff)
{
  ZeroMemory(m_factors,sizeof(m_factors));
}

BlendState::BlendState(ID3D11BlendState* state,float factors[4],UINT mask)
:m_state(state),
 m_mask(mask)
{
  memcpy(m_factors,factors,sizeof(m_factors));
}

BlendState::~BlendState()
{
  if (m_state)
  {
    m_state->Release();
  }
}

void BlendState::MakeActive(ID3D11DeviceContext* context)
{
  context->OMSetBlendState(m_state,m_factors,m_mask);
}
