#include "Graphics/Buffers/VertexBuffer.h"
#include "log.h"

VertexBuffer::VertexBuffer(ID3D11Buffer* buffer,UINT num)
:m_buffer(buffer),
 m_num(num)
{
}

VertexBuffer::~VertexBuffer()
{
  if (m_buffer)
  {
    m_buffer->Release();
  }
}

void VertexBuffer::MakeActive(ID3D11DeviceContext* context)
{
  UINT stride = GetStride();
  UINT offset = 0;
  context->IASetVertexBuffers(0,1,&m_buffer,&stride,&offset);
}

UINT VertexBuffer::GetNumEntries() const
{
  return m_num;
}

ID3D11Buffer* VertexBuffer::CreateBuffer(D3D11_Core& graphics,UINT num,
  UINT size,bool host_write,const void* data)
{
  ID3D11Buffer* back = NULL;
  
  ID3D11Device* device = graphics.GetDevice();
  
  D3D11_BUFFER_DESC bd;
  ZeroMemory(&bd,sizeof(bd));
  if (host_write)
  {
    bd.Usage          = D3D11_USAGE_DYNAMIC;
    bd.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
  }
  else
  {
    bd.Usage          = D3D11_USAGE_DEFAULT;
    bd.CPUAccessFlags = 0;
  }
  bd.ByteWidth = num * size;
  bd.BindFlags = D3D11_BIND_VERTEX_BUFFER;
  D3D11_SUBRESOURCE_DATA res_data;
  ZeroMemory(&res_data,sizeof(res_data));
  res_data.pSysMem = data;
  if (FAILED(device->CreateBuffer(&bd,&res_data,&back)))
  {
    log_print("Failed to create vertex buffer");
    return NULL;
  }
  
  return back;
}

ID3D11Buffer* VertexBuffer::GetBuffer()
{
  return m_buffer;
}