#include "Graphics/DepthStencilState.h"
#include "log.h"

DepthStencilState* DepthStencilState::Create(D3D11_DEPTH_STENCIL_DESC desc,
  D3D11_Core& graphics)
{
  ID3D11DepthStencilState* state = NULL;
  if (FAILED(graphics.GetDevice()->CreateDepthStencilState(&desc,&state)))
  {
    log_print("Failed to create depth stencil state\n");
    return NULL;
  }
  
  return new DepthStencilState(state);
}

DepthStencilState* DepthStencilState::GetCurrent(ID3D11DeviceContext* context)
{
  ID3D11DepthStencilState* state = NULL;
  UINT                     ref   = 0;
  context->OMGetDepthStencilState(&state,&ref);
  
  return new DepthStencilState(state,ref);
}

DepthStencilState::DepthStencilState(ID3D11DepthStencilState* state)
:m_state(state),
 m_ref(1)
{
}

DepthStencilState::DepthStencilState(ID3D11DepthStencilState* state,UINT ref)
:m_state(state),
 m_ref(ref)
{
}

DepthStencilState::~DepthStencilState()
{
  if (m_state)
  {
    m_state->Release();
  }
}

void DepthStencilState::MakeActive(ID3D11DeviceContext* context)
{
  context->OMSetDepthStencilState(m_state,m_ref);
}
