#include "Graphics/Shaders/InputLayout.h"

InputLayout::InputLayout(UINT num)
:m_num(num),
 m_next(0)
{
  m_layout = new D3D11_INPUT_ELEMENT_DESC[num];
}

InputLayout::~InputLayout()
{
  delete m_layout;
}

bool InputLayout::SetNextElement(Semantics semantic,UINT index,
  DXGI_FORMAT format,bool instance)
{
  if (m_next >= m_num)
  {
    return false;
  }
  
  m_layout[m_next].SemanticName           = GetSemanticName(semantic);
  m_layout[m_next].SemanticIndex          = index;
  m_layout[m_next].Format                 = format;
  m_layout[m_next].AlignedByteOffset      = D3D11_APPEND_ALIGNED_ELEMENT;
  
  if (instance)
  {
    m_layout[m_next].InputSlot            = 1;
    m_layout[m_next].InputSlotClass       = D3D11_INPUT_PER_INSTANCE_DATA;
    m_layout[m_next].InstanceDataStepRate = 1;
  }
  else
  {
    m_layout[m_next].InputSlot            = 0;
    m_layout[m_next].InputSlotClass       = D3D11_INPUT_PER_VERTEX_DATA;
    m_layout[m_next].InstanceDataStepRate = 0;
  }
  
  ++m_next;
  
  return true;
}

UINT InputLayout::GetNum() const
{
  return m_num;
}

const D3D11_INPUT_ELEMENT_DESC* InputLayout::GetLayout() const
{
  return m_layout;
}
