#include "Graphics/Shaders/PixelShader.h"
using namespace std;

PixelShader::PixelShader(ID3D11PixelShader* pixel,
  const vector<ID3D11Buffer*>& buffers)
:ShaderBase(buffers),
 m_pixel(pixel)
{
  ZeroMemory(m_textures,sizeof(m_textures));
  ZeroMemory(m_samplers,sizeof(m_samplers));
}

PixelShader::~PixelShader()
{
  for (int i = 0; i < D3D11_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT; ++i)
  {
    if (m_textures[i])
    {
      m_textures[i]->Release();
    }
  }
  
  for (int i = 0; i < D3D11_COMMONSHADER_SAMPLER_SLOT_COUNT; ++i)
  {
    if (m_samplers[i])
    {
      m_samplers[i]->Release();
    }
  }
  
  m_pixel->Release();
}

void PixelShader::MakeActive(ID3D11DeviceContext* context)
{
  // make the pixel shader active
  context->PSSetShader(m_pixel,NULL,0);
  
  // send the constant buffers
  context->PSSetConstantBuffers(0,
    D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT,GetBuffers());
  
  // send the textures
  context->PSSetShaderResources(0,D3D11_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT,
    m_textures);
  
  // send the samplers
  context->PSSetSamplers(0,D3D11_COMMONSHADER_SAMPLER_SLOT_COUNT,m_samplers);
}

void PixelShader::UpdateConstantBuffers(ID3D11DeviceContext* context)
{
  // send the constant buffers
  context->PSSetConstantBuffers(0,
    D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT,GetBuffers());
}

void PixelShader::UseTexture(Texture* tex,UINT index)
{
  // allow the previous value to be cleaned up
  if (m_textures[index])
  {
    m_textures[index]->Release();
  }
  
  // store the new value
  ID3D11ShaderResourceView* tmp = tex == NULL ? NULL : tex->GetTexture();
  m_textures[index] = tmp;
  
  // make sure the new value is not dealloced while this is using it
  if (tmp)
  {
    tmp->AddRef();
  }
}

void PixelShader::UseTextureSampler(TextureSampler* sampler,UINT index)
{
  // allow the previous value to be cleaned up
  if (m_samplers[index])
  {
    m_samplers[index]->Release();
  }
  
  // store the new value
  ID3D11SamplerState* tmp = sampler->GetTextureSampler();
  m_samplers[index] = tmp;
  
  // make sure the new value is not dealloced while this is using it
  if (tmp)
  {
    tmp->AddRef();
  }
}
