#include <vector>
#include "GameMain.h"
#include <directxmath.h>
#include "Graphics/VertexDeclarations/Vertex_PositionTexture.h"
using namespace std;
using namespace DirectX;

GameMain::GameMain(WCHAR* title)
:Game(title),
 m_vertex_shader(NULL),
 m_pixel_shader(NULL),
 m_verts(NULL),
 m_indices(NULL),
 m_instance(NULL),
 m_texture(NULL),
 m_sampler(NULL),
 m_vert_array(NULL)
{
}

GameMain::~GameMain()
{
}

void GameMain::LoadContent()
{
  D3D11_Core& graphics = GetGraphics();
  ContentManager& content = GetContentManager();
  
  vector<UINT> buffer_widths;
  buffer_widths.push_back(sizeof(XMMATRIX));
  
  InputLayout layout(3);
  layout.SetNextElement(SEM_POSITION,0,DXGI_FORMAT_R32G32B32_FLOAT,false);
  layout.SetNextElement(SEM_TEXCOORD,0,DXGI_FORMAT_R32G32_FLOAT,   false);
  layout.SetNextElement(SEM_TEXCOORD,1,DXGI_FORMAT_R32G32B32_FLOAT,true );
  
  m_vertex_shader = content.LoadVertexShader(L"basic_shader_vs.cso",layout,
    buffer_widths);
  if (m_vertex_shader == NULL)
  {
    exit(1);
  }
  
  m_pixel_shader = content.LoadPixelShader(L"basic_shader_ps.cso",
    buffer_widths);
  if (m_pixel_shader == NULL)
  {
    exit(1);
  }
  
  // setup the viewports
  D3D11_VIEWPORT full_viewport = graphics.GetDefaultViewport();
  D3D11_VIEWPORT section_viewport = full_viewport;
  section_viewport.Width /= 2;
  section_viewport.Height /= 2;
  m_viewports.SetNextViewport(section_viewport);
  section_viewport.TopLeftX = section_viewport.Width;
  m_viewports.SetNextViewport(section_viewport);
  section_viewport.TopLeftX = 0;
  section_viewport.TopLeftY = section_viewport.Height;
  m_viewports.SetNextViewport(section_viewport);
  section_viewport.TopLeftX = section_viewport.Width;
  m_viewports.SetNextViewport(section_viewport);
  
  // setup the cameras for the viewports
  m_cameras[0] = new Camera(full_viewport.Width / full_viewport.Height,
    XMFLOAT4(0,0,-10,1),XMFLOAT4(0,0,1,0),XMFLOAT4(0,1,0,0));
  m_cameras[1] = new Camera(full_viewport.Width / full_viewport.Height,
    XMFLOAT4(10,0,0,1),XMFLOAT4(-1,0,0,0),XMFLOAT4(0,1,0,0));
  m_cameras[2] = new Camera(full_viewport.Width / full_viewport.Height,
    XMFLOAT4(-10,0,0,1),XMFLOAT4(1,0,0,0),XMFLOAT4(0,1,0,0));
  m_cameras[3] = new Camera(full_viewport.Width / full_viewport.Height,
    XMFLOAT4(0,10,0,1),XMFLOAT4(0,-1,0,0),XMFLOAT4(0,0,-1,0));
  
  // create the vertex buffer
  Vertex_PositionTexture vertices[] =
  {
    { XMFLOAT3(-1.0f, 1.0f,-1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f, 1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f, 1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3(-1.0f,-1.0f,-1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f, 1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f,-1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3(-1.0f,-1.0f, 1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3(-1.0f,-1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3(-1.0f, 1.0f,-1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f, 1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3( 1.0f,-1.0f, 1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f,-1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3( 1.0f, 1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3(-1.0f,-1.0f,-1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f,-1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f,-1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f, 1.0f,-1.0f),XMFLOAT2(0.0f,1.0f) },

    { XMFLOAT3(-1.0f,-1.0f, 1.0f),XMFLOAT2(0.0f,0.0f) },
    { XMFLOAT3( 1.0f,-1.0f, 1.0f),XMFLOAT2(1.0f,0.0f) },
    { XMFLOAT3( 1.0f, 1.0f, 1.0f),XMFLOAT2(1.0f,1.0f) },
    { XMFLOAT3(-1.0f, 1.0f, 1.0f),XMFLOAT2(0.0f,1.0f) },
  };
  m_verts = VertexBuffer_PositionTexture::Create(graphics,
    ARRAYSIZE(vertices),vertices);
  if (m_verts == NULL)
  {
    exit(1);
  }
  
  // create the index buffer
  WORD indices[] =
  {
    3,1,0,
    2,1,3,

    6,4,5,
    7,4,6,

    11,9,8,
    10,9,11,

    14,12,13,
    15,12,14,

    19,17,16,
    18,17,19,

    22,20,21,
    23,20,22
  };
  m_indices = IndexBuffer16::Create(graphics,ARRAYSIZE(indices),indices);
  if (m_indices == NULL)
  {
    exit(1);
  }
  
  // create the instance buffer
  XMFLOAT3 instance_pos[] =
  {
    XMFLOAT3(-2,0,0),
    XMFLOAT3( 2,0,0),
  };
  m_instance = InstanceBuffer::Create(graphics,
    sizeof(instance_pos) / sizeof(XMFLOAT3),sizeof(XMFLOAT3),instance_pos);
  if (m_instance == NULL)
  {
    exit(1);
  }
  
  // combine the vertex and instance buffer
  m_vert_array = new VertexBufferArray(2);
  m_vert_array->Set(0,*m_verts);
  m_vert_array->Set(1,*m_instance);

  // load the test texture
  m_texture = content.LoadTexture(L"test.png");
  if (m_texture == NULL)
  {
    exit(1);
  }
  
  // create the texture sampler
  D3D11_SAMPLER_DESC sample_desc;
  ZeroMemory(&sample_desc,sizeof(sample_desc));
  sample_desc.Filter         = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
  sample_desc.AddressU       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.AddressV       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.AddressW       = D3D11_TEXTURE_ADDRESS_WRAP;
  sample_desc.ComparisonFunc = D3D11_COMPARISON_NEVER;
  sample_desc.MinLOD         = 0;
  sample_desc.MaxLOD         = D3D11_FLOAT32_MAX;
  m_sampler = TextureSampler::Create(sample_desc,graphics);
  if (m_sampler == NULL)
  {
    exit(1);
  }
  
  // setup the pixel shader to use the texture and sampler
  m_pixel_shader->UseTexture(m_texture,0);
  m_pixel_shader->UseTextureSampler(m_sampler,0);
}

void GameMain::UnloadContent()
{
  ContentManager& content = GetContentManager();
  
  delete m_instance;
  delete m_sampler;
  delete m_verts;
  delete m_indices;
  delete m_vert_array;
  content.UnloadTexture(m_texture);
  content.UnloadShader(m_vertex_shader);
  content.UnloadShader(m_pixel_shader);
}

void GameMain::Update(UINT ms)
{
  static bool resized = false;
  static bool fullscreen = false;
  
  const KeyboardState& keyboard = GetKeyboardState();
  if (keyboard.IsKeyDown(VK_ESCAPE,false))
  {
    Exit();
  }
  else if (keyboard.IsKeyDown(VK_F1,false) && !resized)
  {
    Resize(1024,768);
    resized = true;
  }
#if 0
  else if (keyboard.IsKeyDown(VK_F2,false) && !fullscreen)
  {
    // enter full screen mode
    Fullscreen(true);
    fullscreen = true;
  }
  else if (keyboard.IsKeyDown(VK_F3,false) && fullscreen)
  {
    // return to window mode
    Fullscreen(false);
    fullscreen = false;
  }
#endif /* 0 */
}

void GameMain::Draw(UINT ms)
{
  D3D11_Core& graphics = GetGraphics();
  
  // only using 1 d3d11 thread, so just get the immediate context
  ID3D11DeviceContext* context = graphics.GetImmediateContext();
  
  // apply the shaders
  m_vertex_shader->MakeActive(context);
  m_pixel_shader->MakeActive(context);
  
  // get the vertex arrays and index array ready for rendering
  m_vert_array->MakeActive(context);
  m_indices->MakeActive(context);
  
  // draw the test model in each viewport
  for (UINT i = 0; i < 4; ++i)
  {
    // make the current viewport active
    m_viewports.MakeActive(context,i,1);
    
    // set the wvp matrix for the viewport
    XMMATRIX tmp;
    XMMATRIX wvp = XMMatrixIdentity();
    m_cameras[i]->GetView(tmp);
    wvp *= tmp;
    m_cameras[i]->GetProj(tmp);
    wvp *= tmp;
    wvp = XMMatrixTranspose(wvp);
    m_vertex_shader->SetConstantBuffer(context,0,&wvp);
    m_vertex_shader->UpdateConstantBuffers(context);
    
    // draw
    graphics.DrawInstancedPrimitives(context,
      D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST,0,36,0,2);
  }
}
