#ifndef GAMEMAIN_H
#define GAMEMAIN_H

// d3d11_framework header files
#include "Game.h"
#include "Graphics/Shaders/VertexShader.h"
#include "Graphics/Shaders/PixelShader.h"
#include "Graphics/Buffers/VertexBuffer_PositionTexture.h"
#include "Graphics/Buffers/IndexBuffer16.h"
#include "Graphics/Buffers/InstanceBuffer.h"
#include "Graphics/Buffers/VertexBufferArray.h"
#include "Graphics/Texture.h"
#include "Graphics/TextureSampler.h"
#include "Graphics/Viewports.h"

// local project header file(s)
#include "Camera.h"

class GameMain : public Game
{
  public:
    GameMain(WCHAR* title);
    ~GameMain();
    
  protected:
    /// <summary>
    /// Loads the content for the game
    /// </summary>
    /// <param name="content">
    /// content manager to use for loading resources
    /// </param>
    void LoadContent();
    
    /// <summary>
    /// Unloads the content for the game before the dealloction of the instance
    /// </summary>
    /// <param name="content">
    /// content manager to use for unloading resources
    /// </param>
    void UnloadContent();
    
    /// <summary>
    /// Updates the game state
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the update
    /// </param>
    void Update(UINT ms);
    
    /// <summary>
    /// Draws the current frame
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the last frame
    /// </param>
    void Draw(UINT ms);
    
  private:
    // disabled
    GameMain(const GameMain& cpy);
    GameMain& operator=(const GameMain& cpy);
    
    /// <summary>
    /// test vertex shader
    /// </summary>
    VertexShader* m_vertex_shader;
    
    /// <summary>
    /// test pixel shader
    /// </summary>
    PixelShader* m_pixel_shader;

    /// <summary>
    /// vertex buffer for the test cube
    /// </summary>
    VertexBuffer_PositionTexture* m_verts;

    /// <summary>
    /// index buffer for the test cube
    /// </summary>
    IndexBuffer16* m_indices;

    /// <summary>
    /// instance buffer for the test cube
    /// </summary>
    InstanceBuffer* m_instance;
    
    /// <summary>
    /// texture to apply to the cube
    /// </summary>
    Texture* m_texture;
    
    /// <summary>
    /// texture sampler to use
    /// </summary>
    TextureSampler* m_sampler;
    
    /// <summary>
    /// vertex buffer array for supplying the vertex and instance buffers to
    /// the rendering process
    /// </summary>
    VertexBufferArray* m_vert_array;

    /// <summary>
    /// cameras for the 4 test viewports
    /// </summary>
    Camera* m_cameras[4];
    
    /// <summary>
    /// viewports to test the viewport functionality
    /// </summary>
    Viewports m_viewports;
};

#endif /* GAMEMAIN_H */
